function data = getData(dataLabel, opts)
switch dataLabel
    case 'toy'
        data = halfDisk(opts);
    case 'digits'
        load('digits.mat');
    otherwise
        error('Invalid dataLabel');
end

function data = halfDisk(opts)
assert(length(opts.mu) == 2);
w = opts.w;
w = w/sqrt(w'*w);

% Training data
x = unifrnd(opts.range(1), opts.range(2), 2, opts.numPoints);
a = w'*x;
valid = abs(a) > opts.margin;
data.train.x = x(:, valid);

% add noise to the data (zero mean Gaussian noise with sigma=opts.noise)
noisex = mvnrnd([0 0], eye(2)*opts.noise, sum(valid))';
data.train.x = data.train.x + noisex;
data.train.y = sign(a(:, valid));

% Testing data
x = unifrnd(opts.range(1), opts.range(2), 2, opts.numPoints);
a = w'*x;
valid = abs(a) > opts.margin;
data.test.x = x(:, valid);
data.test.y = sign(a(:, valid));
